package	jp.co.sds_corp.tool.dba;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

public class ExcelWriter
{
	Vector<ColumnInfo>	vecClm = new Vector<ColumnInfo>();
	String	table;
	String	tmpFile;
	String	tmpSheet;
	HSSFSheet template;
	HSSFWorkbook tmpBook;
	int		nTblRow;
	int		nTblCol;
	int		nStart;
	int[]	nCols = new int[6];

	public ExcelWriter() {
	}

	public void init( IConfiguration config ) throws Exception {
		IConfiguration c, cc;
		cc = config.getChild( "Excel" );
		tmpFile = cc.getAttributes().get( "template" );
		tmpSheet = cc.getAttributes().get( "sheet" );

		c = cc.getChild( "table" ).getAttributes();
		nTblRow = c.getInteger( "row" );
		nTblCol = c.getInteger( "col" );

		c = cc.getChild( "column" ).getAttributes();
		nStart = c.getInteger( "startrow" );
		nCols[0] = c.getInteger( "no" );
		nCols[1] = c.getInteger( "name" );
		nCols[2] = c.getInteger( "column" );
		nCols[3] = c.getInteger( "type" );
		nCols[4] = c.getInteger( "key" );
		nCols[5] = c.getInteger( "desc" );

		readTemplate();
	}

	private void readTemplate() throws Exception {
		FileInputStream is = new FileInputStream( new File( tmpFile ));
		tmpBook = new HSSFWorkbook( is );
		template = tmpBook.getSheet( tmpSheet );
		is.close();
	}

	public void create( String filename ) throws Exception {
		File file = new File( filename );
		String name = file.getName();
		if ( name.indexOf( "." ) < 0 ) {
			throw( new Exception( "Illegal file name:" + filename ));
		}
		table = name.substring( 0, name.indexOf( "." ));
		BufferedReader br = new BufferedReader(
							new InputStreamReader(
								new FileInputStream( filename ) ));
		vecClm.clear();
		int     nLine = 0;
		String  line;
		for ( ;; ) {
			line = br.readLine();
			if ( line == null ) {
				break;
			}
			nLine++;
			// 1行目は読み飛ばし
			if ( nLine == 1 ) {
				continue;
			}
			// 空行は読み飛ばし
			if ( line.length() == 0 ) {
				continue;
			}
			String[] buf = line.split( "\t" );
			vecClm.add( new ColumnInfo( buf ));
		}
		br.close();

		createExcel( table + ".xls" );
	}

	private HSSFCell createCell( HSSFRow row, int nCol ) {
		HSSFCell ret = row.createCell( (short )nCol );
		// HSSFCellStyle style = ret.getCellStyle();
		HSSFCellStyle style = tmpBook.createCellStyle();
		style.setBorderTop( HSSFCellStyle.BORDER_THIN );
		style.setBorderBottom( HSSFCellStyle.BORDER_THIN );
		style.setBorderLeft( HSSFCellStyle.BORDER_THIN );
		style.setBorderRight( HSSFCellStyle.BORDER_THIN );
		ret.setCellStyle( style );
		ret.setEncoding( HSSFCell.ENCODING_UTF_16 );
		return ret;
	}

	protected void createExcel( String file ) throws Exception {
		System.out.println( "create " + table + "..." );
		HSSFRow row;
		HSSFCell cell;
		row = template.getRow( nTblRow );
		row.getCell( (short )nTblCol ).setCellValue( table );

		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			row = template.createRow( nStart + i );

			cell = createCell( row, nCols[0] );
			cell.getCellStyle().setAlignment( HSSFCellStyle.ALIGN_CENTER );
			cell.setCellValue( i + 1 );

			cell = createCell( row, nCols[1] );
			cell.setCellValue( col.getJName() );
			cell = createCell( row, nCols[2] );
			cell.setCellValue( col.getName() );
			cell = createCell( row, nCols[3] );
			cell.setCellValue( col.getFullType() );
			cell = createCell( row, nCols[4] );
			cell.getCellStyle().setAlignment( HSSFCellStyle.ALIGN_CENTER );
			cell.setCellValue( col.isKey() ? "○" : "" );
			cell = createCell( row, nCols[5] );
			cell.setCellValue( col.getDesc() );
		}

		FileOutputStream os = new FileOutputStream( file );
		tmpBook.write( os );
		os.close();
	}

	public static void main( String[] args ) {
		try {
			String file = System.getProperty( "dba.config" );
			DOMConfiguration config = new DOMConfiguration( file );
			ExcelWriter excel = new ExcelWriter();
			excel.init( config );

			for ( int i = 0; i < args.length; i++ ) {
				excel.create( args[i] );
			}
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}

}

