package	jp.co.sds_corp.tool.dba;

import java.io.PrintWriter;

public class TableOracle extends TableGenerator
{
	public TableOracle( String file_name ) throws Exception {
		super( file_name );
	}

	protected void printHeader( PrintWriter out ) {
		out.println( "#!/bin/csh" );
		out.println( "#--------------------------------------------------" );
		out.println( "# テーブル定義スクリプト" );
		out.println( "# " + table );
		out.println( "#--------------------------------------------------" );
		out.println( "" );
		if ( useSqlTool()) {
			out.println( "sql $SQLOPT << !" );
		} else {
			out.println( "sqlplus $DBO/$DBO_PWD << !" );
			out.println( "--       Table name      : " + table );
		}
		out.println( "" );
		out.println( "create table " + table + " (" );
	}

	protected void printColumn( PrintWriter out, ColumnInfo col, int nPos ) {
		out.print( "\t" + col.getName() );
		int nTab;
		nTab = ( 31 - col.getName().length()) / 4;
		for ( int i = 0; i <  nTab; i++ ) {
			out.print( "\t" );
		}
		out.print( col.getFullType() );
		nTab = ( 31 - col.getFullType().length()) / 4;
		for ( int i = 0; i <  nTab; i++ ) {
			out.print( "\t" );
		}
		if ( col.isKey()) {
			out.print( " not null" );
		}
		out.println( ( isEndColumn( nPos ) ? "" : "," ) );
	}

	protected void printFooter( PrintWriter out ) {
		out.print( ")" );
		if ( useSqlTool()) {
			out.println( "\ngo" );
		} else {
			out.println( ";" );
		}
		out.println( "" );

		if ( countKey() > 0 ) {
			out.println( "alter table " + table
					+ " add constraint p_" + table + " primary key (" );
			int nIdx = 0;
			for ( int i = 0; i < vecClm.size(); i++ ) {
				ColumnInfo col = (ColumnInfo )vecClm.get( i );
				if ( !col.isKey()) {
					continue;
				}
				if ( nIdx != 0 ) {
					out.print( "," );
				}
				nIdx++;
				out.print( col.getName() );
			}
			out.print( ")" );
			if ( useSqlTool()) {
				out.println( "\ngo" );
			} else {
				out.println( ";" );
			}
		}
		printComment( out );
		out.println( "!" );
		out.println( "exit $status" );
	}

	private void printComment( PrintWriter out ) {
		out.print( "COMMENT ON TABLE " + table + " IS '" + table + "' " );
		if ( useSqlTool()) {
			out.println( "\ngo" );
		} else {
			out.println( ";" );
		}
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			out.print( "COMMENT ON COLUMN "
						+ table + "." + col.getName()
						+ " IS '" + col.getJName() + "' " );
//						+ " IS '" + col.getDesc() + "' " );
			if ( useSqlTool()) {
				out.println( "\ngo" );
			} else {
				out.println( ";" );
			}
		}
	}

	public static void main( String[] args ) {
		try {
			for ( int i = 0; i < args.length; i++ ) {
				TableOracle tbl = new TableOracle( args[i] );
				tbl.create();
			}
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}
}

