package	jp.co.sds_corp.tool.dba;

import java.io.PrintWriter;

public class TableSybase extends TableGenerator
{
	public TableSybase( String file_name ) throws Exception {
		super( file_name );
	}

	protected void printHeader( PrintWriter out ) {
		out.println( "#!/bin/csh" );
		out.println( "#--------------------------------------------------" );
		out.println( "# テーブル定義スクリプト" );
		out.println( "# " + table );
		out.println( "#--------------------------------------------------" );
		out.println( "" );
		if ( useSqlTool()) {
			out.println( "sql $SQLOPT << !" );
		} else {
			out.println( "isql -U$DBO -P $DBO_PWD << !" );
		}
		out.println( "/**************** define table ***********************" );
		out.println( "*       Table name      : " + table );
		out.println( "*****************************************************/" );
		out.println( "" );
		out.println( "use $argv[1]" );
		out.println( "go" );
		out.println( "" );
		out.println( "if (exists (select * from sysobjects where name = '"
						+ table + "')) " );
		out.println( "\tdrop table " + table );
		out.println( "go" );
		out.println( "" );
		out.println( "create table " + table + " (" );
	}

	protected void printColumn( PrintWriter out, ColumnInfo col, int nPos ) {
		out.print( "\t" + col.getName() );
		int nTab;
		nTab = ( 31 - col.getName().length()) / 4;
		for ( int i = 0; i <  nTab; i++ ) {
			out.print( "\t" );
		}
		// Oracleの定義で作成する場合
		if ( col.getType().equals( "varchar2" )) {
			col.setType( "varchar" );
		}
		if ( col.getType().equals( "timestamp" )) {
			col.setType( "datetime" );
			col.setSize( 0 );
		}
		if ( col.getType().equals( "number" )) {
			col.setType( "decimal" );
		}
		// Oracleの定義で作成する場合 ここまで
		out.print( col.getFullType() );
		nTab = ( 31 - col.getFullType().length()) / 4;
		for ( int i = 0; i <  nTab; i++ ) {
			out.print( "\t" );
		}
		out.println( " not null" + ( isEndColumn( nPos ) ? "" : "," ) );
	}

	protected void printFooter( PrintWriter out ) {
		out.println( ") lock datarows" );
		out.println( "go" );
		out.println( "" );
		out.println( "grant all on " + table + " to public" );
		out.println( "go" );

		if ( countKey() > 0 ) {
			out.println( "create unique clustered index" );
			out.println( table + "_idx on " + table + "(" );
			int nIdx;
			nIdx = 0;
			for ( int i = 0; i < vecClm.size(); i++ ) {
				ColumnInfo col = (ColumnInfo )vecClm.get( i );
				if ( !col.isKey()) {
					continue;
				}
				if ( nIdx != 0 ) {
					out.print( "," );
				}
				nIdx++;
				out.print( col.getName() );
			}
			out.println( "\n)" );
			out.println( "go" );
			out.print( "sp_primarykey " + table + "," );
			nIdx = 0;
			for ( int i = 0; i < vecClm.size(); i++ ) {
				ColumnInfo col = (ColumnInfo )vecClm.get( i );
				if ( !col.isKey()) {
					continue;
				}
				if ( nIdx != 0 ) {
					out.print( "," );
				}
				nIdx++;
				out.print( col.getName() );
			}
			out.println( "\ngo" );
		}
		out.println( "!" );
		out.println( "exit $status" );
	}

	public static void main( String[] args ) {
		try {
			for ( int i = 0; i < args.length; i++ ) {
				TableSybase tbl = new TableSybase( args[i] );
				tbl.create();
			}
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}
}

