package	jp.co.sds_corp.tool.http;

import	java.io.BufferedReader;
import	java.io.InputStream;
import	java.io.InputStreamReader;
import	java.io.IOException;
import	java.io.OutputStream;
import	java.io.PrintWriter;
import	java.net.CookieHandler;
import	java.net.CookieManager;
import	java.net.HttpCookie;
import	java.net.HttpURLConnection;
import	java.net.URL;
import	java.net.URLEncoder;
import	java.util.HashMap;
import	java.util.List;
import	java.util.Map;
import	java.util.regex.Pattern;
import	java.util.regex.Matcher;

import jp.co.sds_corp.util.log.CmnLogFactory;
import	jp.co.sds_corp.util.log.CmnLogger;

public class HttpRequester
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	Map<String,String> mapResult = new HashMap<String,String>();
	Pattern pattern = Pattern.compile( "\\$\\{([\\w.]+)\\}" );

	public void request( RequestInfo info ) throws Exception {
		URL url;
		HttpURLConnection con = null;
		logger.debug( "request url=" + info.getUrl()
			+ " keepAlive=" + info.isKeepAlive()
			+ " followRedirects=" + info.isFollowRedirects() );
		try {
			url = new URL( info.getUrl() );
			con = (HttpURLConnection )url.openConnection();
			con.setRequestMethod( info.getMethod() );
			con.setDoInput( true );
			con.setDoOutput( true );
			if ( info.isFollowRedirects()) {
				HttpURLConnection.setFollowRedirects( true );
			}
			if ( info.isKeepAlive()) {
				con.setRequestProperty( "Connection", "Keep-Alive" );
			}
			setHeader( con, info.mapHeader() );
			setRequest( con, info.mapRequest() );

			con.connect();
			printDebug( con );

			int resCode = con.getResponseCode();
			if ( resCode == HttpURLConnection.HTTP_OK ) {
				parseResponse( info, con );
			} else {
				logger.warn( "HTTP error " + con.getResponseCode()
							+ " " + con.getResponseMessage() );
			}
		} finally {
			if ( con != null && !info.isKeepAlive()) {
				con.disconnect();
			}
		}
	}

	public void close() {
	}

	void setHeader( HttpURLConnection con, Map<String,String> map )
		throws IOException 
	{
		for (Map.Entry<String,String> entry : map.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			value = replaceValue( value );
			logger.trace( "set header " + key + "=" + value );
			con.setRequestProperty( key, value );
		}
	}

	void setRequest( HttpURLConnection con, Map<String,String> map )
		throws IOException 
	{
		OutputStream os = con.getOutputStream();
		PrintWriter pw = new PrintWriter( os );
		int pos = 0;
		for (Map.Entry<String,String> entry : map.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			value = replaceValue( value );
			value = URLEncoder.encode( value, "UTF-8" );
			logger.trace( "set request " + key + "=" + value );
			if ( pos > 0 ) {
				pw.print( "&" );
			}
			pos++;
			pw.print( key + "=" + value );
		}
		pw.flush();
		pw.close();
		os.close();
	}

	protected String replaceValue( String value ) {
		if ( value == null ) {
			return null;
		}
		Matcher matcher = pattern.matcher( value );
		if ( !matcher.find()) {
			return value;
		}
		String key = matcher.group( 1 );
		String var = mapResult.get( key );
		if ( var == null ) {
			var = System.getProperty( key );
			if ( var == null ) {
				return value;
			}
		}
		String ret = matcher.replaceFirst( var );
		return replaceValue( ret );
	}

	void parseResponse( RequestInfo info, HttpURLConnection con )
		throws Exception
	{
		String response = readResponse( con );
		for ( ResponseParser parser : info.listParser()) {
			String res = parser.parse( response );
			if ( res != null ) {
				mapResult.put( parser.getName(), res );
			}
		}
	}

	String readResponse( HttpURLConnection con ) throws Exception {
		StringBuilder sb = new StringBuilder();
		InputStream is = con.getInputStream();
		BufferedReader br = new BufferedReader( new InputStreamReader( is ));
		for ( ;; ) {
			String line = br.readLine();
			if ( line == null ) {
				break;
			}
			sb = sb.append( line + "\n" );
		}
		br.close();
		is.close();
		return sb.toString();
	}

	public Map<String,String> getResult() {
		return mapResult;
	}

	public void setResult( Map<String,String> var ) {
		this.mapResult = var;
	}

	void printDebug( HttpURLConnection con ) throws IOException {
		logger.debug( "ResponseCode=" + con.getResponseCode() );
		logger.debug( "ResponseMessage=" + con.getResponseMessage() );
		Map<String, List<String>> header = con.getHeaderFields();
		for (Map.Entry<String,List<String>> entry : header.entrySet()) {
			String key = entry.getKey();
			List<String> value = entry.getValue();
			logger.trace( "@@@ Header " + key + ": " + value);
		}
		CookieManager manager = (CookieManager )CookieHandler.getDefault();
		List<HttpCookie> list = manager.getCookieStore().getCookies();
		for ( HttpCookie cookie : list ) {
			logger.trace( "@@@ Cookie " + cookie + " "
				+ cookie.getPath() + ":" + cookie.getDomain() );
		}
		logger.trace( "cookie size=" + list.size() );
	}

}
