package jp.co.sds_corp.tool;

import java.util.List;
import java.util.regex.Pattern;

import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.tool.check.Checker;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class ClassFinderTest extends BasicTestCase
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testFile1() throws Exception {
		ClassFinder<Checker> finder = new ClassFinder<Checker>();
		List<Class<Checker>> list = finder.findClass("jp.co.sds_corp.tool");
		for (Class<Checker> clazz : list) {
			logger.debug( "class=" + clazz);
			assertTrue("isChecker", Checker.class.isAssignableFrom(clazz));
		}
	}

	public void testFile2() throws Exception {
		String[] excludes = {".*Java.*"};
		Pattern pattern = Pattern.compile(excludes[0]);
		ClassFinder<Checker> finder = new ClassFinder<Checker>();
		List<Class<Checker>> list = finder.findClass("jp.co.sds_corp.tool", excludes);
		for (Class<Checker> clazz : list) {
			logger.debug( "class=" + clazz);
			assertTrue("isChecker", Checker.class.isAssignableFrom(clazz));
			assertFalse("pattern", pattern.matcher(clazz.getName()).matches());
		}
	}

	public void testFile3() throws Exception {
		ClassFinder finder = new ClassFinder();
		List<Class<?>> list = finder.findClass("jp.co.sds_corp.tool");
		for (Class<?> clazz : list) {
			logger.debug( "class=" + clazz);
		}
	}

	public void testFile4() throws Exception {
		ClassFinder finder = new ClassFinder(Thread.currentThread().getContextClassLoader());
		List<Class<Checker>> list = finder.findClass("jp.co.sds_corp.tool", Checker.class);
		for (Class<Checker> clazz : list) {
			logger.debug( "class=" + clazz);
		}
		assertEquals("found.size", 4, list.size());
	}

	public void testJar1() throws Exception {
		ClassFinder<IConfigurable> finder = new ClassFinder<IConfigurable>();
		List<Class<IConfigurable>> list = finder.findClass("jp.co.sds_corp.util");
		for (Class<IConfigurable> clazz : list) {
			logger.debug( "class=" + clazz);
			assertTrue("isIConfigurable", IConfigurable.class.isAssignableFrom(clazz));
		}
	}

	public void testJar2() throws Exception {
		String[] excludes = {"jp\\.co\\.sds_corp\\.util\\..*"};
		ClassFinder<IConfigurable> finder = new ClassFinder<IConfigurable>();
		List<Class<IConfigurable>> list = finder.findClass("jp.co.sds_corp.util", excludes);
		for (Class<IConfigurable> clazz : list) {
			logger.debug( "class=" + clazz);
		}
		assertEquals("found.size", 0, list.size());
	}

	public void testFileErr1() throws Exception {
		ClassFinder<Checker> finder = new ClassFinder<Checker>();
		List<Class<Checker>> list = finder.findClass("aaa.bbb.ccc");
		for (Class<Checker> clazz : list) {
			logger.debug( "class=" + clazz);
		}
		assertEquals("found.size", 0, list.size());
	}

}
