package jp.co.sds_corp.tool.log;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.tool.Where;

public class AccessLogTest extends BasicTestCase
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	AccessLogViewer obj;

	public void setUp() throws Exception {
		super.setUp();
		obj = new AccessLogViewer();
	}

	public void testTest1() throws Exception {
		String[] args = { "-where:status>400", "target/test-classes/access_log" };
		AccessLogViewer.main( args );
	}

	public void testParse1() throws Exception {
		logger.debug("testParse1");

		String line = "HOST LOG USER [DATE] \"REQ\" 200 BYTES TIME +";
		AccessLogViewer.Record rec = obj.parseLine(line);
		assertEquals("host", "HOST", rec.host);
		assertEquals("log ", "LOG", rec.log);
		assertEquals("user", "USER", rec.user);
		assertEquals("date", "DATE", rec.date);
		assertEquals("req ", "REQ", rec.request);
		assertEquals("stat", "200", rec.status);
		assertEquals("byte", "BYTES", rec.bytes);
		assertEquals("time", "TIME", rec.time);
		assertEquals("con ", "+", rec.connection);
	}
		
	public void testParseErr1() throws Exception {
		logger.debug("testParseErr1");

		String line = "HOST LOG USER [DATE] \"REQ\" 200 BYTES TIME";
		AccessLogViewer.Record rec = obj.parseLine(line);
		assertEquals("Record ", (AccessLogViewer.Record )null, rec);
	}

	public void testMatch1() throws Exception {
		logger.debug("testParse1");

		Where where;
		String line = "HOST LOG USER [DATE] \"REQ\" 200 BYTES TIME +";
		AccessLogViewer.Record rec = obj.parseLine(line);

		where = new Where("host=HOST");
		assertEquals("host", true, obj.matchWhere(rec, where));
		where = new Where("log=LOG");
		assertEquals("user", true, obj.matchWhere(rec, where));
		where = new Where("user=USER");
		assertEquals("host", true, obj.matchWhere(rec, where));
		where = new Where("date=DATE");
		assertEquals("host", true, obj.matchWhere(rec, where));
		where = new Where("request=REQ");
		assertEquals("host", true, obj.matchWhere(rec, where));
		where = new Where("status=200");
		assertEquals("host", true, obj.matchWhere(rec, where));
		where = new Where("bytes=100");
		assertEquals("host", false, obj.matchWhere(rec, where));
		where = new Where("time=TIME");
		assertEquals("host", true, obj.matchWhere(rec, where));
		where = new Where("connection=+");
		assertEquals("host", true, obj.matchWhere(rec, where));
		where = new Where("FOO=BAR");
		assertEquals("host", false, obj.matchWhere(rec, where));
	}

}
